@@ -6,6 +6,8 @@ const Lang = imports.lang;
 const Shell = imports.gi.Shell;
 const Mainloop = imports.mainloop;
 const St = imports.gi.St;
+const Gettext = imports.gettext.domain('gnome-shell');
+const _ = Gettext.gettext;
 
 const Main = imports.ui.main;
 const MessageTray = imports.ui.messageTray;
@@ -80,6 +82,7 @@ const rewriteRules = {
           replacement: '$2 &lt;$1&gt;' }
     ]
 };
+
 function NotificationDaemon() {
     this._init();
 }
@@ -98,7 +101,13 @@ NotificationDaemon.prototype = {
                                   Lang.bind(this, this._acquiredName),
                                   Lang.bind(this, this._lostName));
 
-        this._currentNotifications = {};
+        this._sources = {};
+        this._senderToPid = {};
+        this._notifications = {};
+        this._busProxy = new Bus();
+
+        Main.statusIconDispatcher.connect('message-icon-added', Lang.bind(this, this._onTrayIconAdded));
+        Main.statusIconDispatcher.connect('message-icon-removed', Lang.bind(this, this._onTrayIconRemoved));
 
         Shell.WindowTracker.get_default().connect('notify::focus-app',
             Lang.bind(this, this._onFocusAppChanged));
@@ -131,18 +140,59 @@ NotificationDaemon.prototype = {
         }
     },
 
-    _sourceId: function(id) {
-        return 'source-' + id;
+    _iconForNotificationData: function(icon, hints, size) {
+        let textureCache = St.TextureCache.get_default();
+
+        if (icon) {
+            if (icon.substr(0, 7) == 'file://')
+                return textureCache.load_uri_async(icon, size, size);
+            else if (icon[0] == '/') {
+                let uri = GLib.filename_to_uri(icon, null);
+                return textureCache.load_uri_async(uri, size, size);
+            } else
+                return textureCache.load_icon_name(icon, St.IconType.FULLCOLOR, size);
+        } else if (hints.icon_data) {
+            let [width, height, rowStride, hasAlpha,
+                 bitsPerSample, nChannels, data] = hints.icon_data;
+            return textureCache.load_from_raw(data, data.length, hasAlpha,
+                                              width, height, rowStride, size);
+        } else {
+            let stockIcon;
+            switch (hints.urgency) {
+                case Urgency.LOW:
+                case Urgency.NORMAL:
+                    stockIcon = 'gtk-dialog-info';
+                    break;
+                case Urgency.CRITICAL:
+                    stockIcon = 'gtk-dialog-error';
+                    break;
+            }
+            return textureCache.load_icon_name(stockIcon, St.IconType.FULLCOLOR, size);
+        }
+    },
+
+    _newSource: function(title, pid) {
+        let source = new Source(title, pid);
+        this._sources[pid] = source;
+
+        source.connect('destroy', Lang.bind(this,
+            function() {
+                delete this._sources[pid];
+            }));
+
+        Main.messageTray.add(source);
+        return source;
     },
 
     Notify: function(appName, replacesId, icon, summary, body,
                      actions, hints, timeout) {
-        let source = Main.messageTray.getSource(this._sourceId(appName));
-        let id = null;
+        let id;
 
         // Filter out notifications from Empathy, since we
         // handle that information from telepathyClient.js
         if (appName == 'Empathy') {
+            // Ignore replacesId since we already sent back a
+            // NotificationClosed for that id.
             id = nextNotificationId++;
             Mainloop.idle_add(Lang.bind(this,
                                         function () {
@@ -151,31 +201,6 @@ NotificationDaemon.prototype = {
             return id;
         }
 
-        hints = Params.parse(hints, { urgency: Urgency.NORMAL }, true);
-
-        // Source may be null if we have never received a notification
-        // from this app or if all notifications from this app have
-        // been acknowledged.
-        if (source == null) {
-            source = new Source(this._sourceId(appName), icon, hints);
-            Main.messageTray.add(source);
-
-            source.connect('clicked', Lang.bind(this,
-                function() {
-                    source.destroy();
-                }));
-
-            let sender = DBus.getCurrentMessageContext().sender;
-            let busProxy = new Bus();
-            busProxy.GetConnectionUnixProcessIDRemote(sender, function (result, excp) {
-                let app = Shell.WindowTracker.get_default().get_app_from_pid(result);
-                if (app)
-                    source.setApp(app);
-            });
-        } else {
-            source.update(icon, hints);
-        }
-
         summary = GLib.markup_escape_text(summary, -1);
 
         let rewrites = rewriteRules[appName];
@@ -187,43 +212,105 @@ NotificationDaemon.prototype = {
             }
         }
 
-        let notification;
-        if (replacesId != 0) {
-            id = replacesId;
-            notification = this._currentNotifications[id];
+        hints = Params.parse(hints, { urgency: Urgency.NORMAL }, true);
+
+        let ndata = { appName: appName,
+                      icon: icon,
+                      summary: summary,
+                      body: body,
+                      actions: actions,
+                      hints: hints,
+                      timeout: timeout };
+        if (replacesId != 0 && this._notifications[replacesId]) {
+            ndata.id = id = replacesId;
+            ndata.notification = this._notifications[replacesId].notification;
+        } else {
+            replacesId = 0;
+            ndata.id = id = nextNotificationId++;
+        }
+        this._notifications[id] = ndata;
+
+        let sender = DBus.getCurrentMessageContext().sender;
+        let pid = this._senderToPid[sender];
+        let source = pid ? this._sources[pid] : null;
+
+        if (source) {
+            this._notifyForSource(source, ndata);
+            return id;
         }
 
+        if (replacesId) {
+            // There's already a pending call to GetConnectionUnixProcessID,
+            // which will see the new notification data when it finishes,
+            // so we don't have to do anything.
+            return id;
+        }
+
+        this._busProxy.GetConnectionUnixProcessIDRemote(sender, Lang.bind(this,
+            function (pid, ex) {
+                // The app may have updated or removed the notification
+                ndata = this._notifications[id];
+                if (!ndata)
+                    return;
+
+                this._senderToPid[sender] = pid;
+                source = this._sources[pid];
+
+                if (!source)
+                    source = this._newSource(appName, pid);
+                source.connect('destroy', Lang.bind(this,
+                    function() {
+                        delete this._senderToPid[sender];
+                    }));
+
+                this._notifyForSource(source, ndata);
+            }));
+
+        return id;
+    },
+
+    _notifyForSource: function(source, ndata) {
+        let [id, icon, summary, body, actions, hints, notification] =
+            [ndata.id, ndata.icon, ndata.summary, ndata.body,
+             ndata.actions, ndata.hints, ndata.notification];
+
+        let iconActor = this._iconForNotificationData(icon, hints, source.ICON_SIZE);
+
         if (notification == null) {
-            id = nextNotificationId++;
-            notification = new MessageTray.Notification(id, source, summary, body, true);
-            this._currentNotifications[id] = notification;
-            notification.connect('dismissed', Lang.bind(this,
+            notification = new MessageTray.Notification(source, summary, body, { icon: iconActor });
+            ndata.notification = notification;
+            notification.connect('clicked', Lang.bind(this,
                 function(n) {
-                    n.destroy();
-                    this._emitNotificationClosed(n.id, NotificationClosedReason.DISMISSED);
+                    this._emitNotificationClosed(id, NotificationClosedReason.DISMISSED);
                 }));
+            notification.connect('destroy', Lang.bind(this,
+                function(n) {
+                    delete this._notifications[id];
+                }));
+            notification.connect('action-invoked', Lang.bind(this, this._actionInvoked, source, id));
         } else {
-            // passing in true as the last parameter will clear out extra actors,
-            // such as actions
-            notification.update(summary, body, true);
+            notification.update(summary, body, { icon: iconActor,
+                                                 clear: true });
         }
 
         if (actions.length) {
             for (let i = 0; i < actions.length - 1; i += 2)
                 notification.addButton(actions[i], actions[i + 1]);
-            notification.connect('action-invoked', Lang.bind(this, this._actionInvoked, source, id));
         }
 
         notification.setUrgent(hints.urgency == Urgency.CRITICAL);
 
-        source.notify(notification);
-        return id;
+        let sourceIconActor = source.useNotificationIcon ? this._iconForNotificationData(icon, hints, source.ICON_SIZE) : null;
+        source.notify(notification, sourceIconActor);
     },
 
     CloseNotification: function(id) {
-        let notification = this._currentNotifications[id];
-        if (notification)
-            notification.destroy();
+        let ndata = this._notifications[id];
+        if (ndata) {
+            if (ndata.notification)
+                ndata.notification.destroy();
+            delete this._notifications[id];
+        }
         this._emitNotificationClosed(id, NotificationClosedReason.APP_CLOSED);
     },
 
@@ -235,8 +322,10 @@ NotificationDaemon.prototype = {
             // 'body-images',
             'body-markup',
             // 'icon-multi',
-            'icon-static'
+            'icon-static',
+            'persistence',
             // 'sound',
+            'x-gnome-icon-buttons'
         ];
     },
 
@@ -251,17 +340,24 @@ NotificationDaemon.prototype = {
 
     _onFocusAppChanged: function() {
         let tracker = Shell.WindowTracker.get_default();
-        if (tracker.focus_app)
-            Main.messageTray.removeSourceByApp(tracker.focus_app);
+        if (!tracker.focus_app)
+            return;
+
+        for (let id in this._sources) {
+            let source = this._sources[id];
+            if (source.app == tracker.focus_app) {
+                source.activated();
+                return;
+            }
+        }
     },
 
     _actionInvoked: function(notification, action, source, id) {
+        source.activated();
         this._emitActionInvoked(id, action);
-        source.destroy();
     },
 
     _emitNotificationClosed: function(id, reason) {
-        delete this._currentNotifications[id];
         DBus.session.emit_signal('/org/freedesktop/Notifications',
                                  'org.freedesktop.Notifications',
                                  'NotificationClosed', 'uu',
@@ -273,85 +369,89 @@ NotificationDaemon.prototype = {
                                  'org.freedesktop.Notifications',
                                  'ActionInvoked', 'us',
                                  [id, action]);
+    },
+
+    _onTrayIconAdded: function(o, icon) {
+        let source = this._sources[icon.pid];
+        if (!source)
+            source = this._newSource(icon.title || icon.wm_class || _("Unknown"), icon.pid);
+        source.setTrayIcon(icon);
+    },
+
+    _onTrayIconRemoved: function(o, icon) {
+        let source = this._sources[icon.pid];
+        if (source)
+            source.destroy();
     }
 };
 
 DBus.conformExport(NotificationDaemon.prototype, NotificationDaemonIface);
 
-function Source(sourceId, icon, hints) {
-    this._init(sourceId, icon, hints);
+function Source(title, pid) {
+    this._init(title, pid);
 }
 
 Source.prototype = {
     __proto__:  MessageTray.Source.prototype,
 
-    _init: function(sourceId, icon, hints) {
-        MessageTray.Source.prototype._init.call(this, sourceId);
+    _init: function(title, pid) {
+        MessageTray.Source.prototype._init.call(this, title);
 
-        this.app = null;
-        this._openAppRequested = false;
-
-        this.update(icon, hints);
+        this._pid = pid;
+        this._setApp();
+        if (this.app)
+            this.title = this.app.get_name();
+        else
+            this.useNotificationIcon = true;
+        this._isTrayIcon = false;
     },
 
-    update: function(icon, hints) {
-        this._icon = icon;
-        this._iconData = hints.icon_data;
-        this._urgency = hints.urgency;
+    notify: function(notification, icon) {
+        if (!this.app)
+            this._setApp();
+        if (!this.app && icon)
+            this._setSummaryIcon(icon);
+        MessageTray.Source.prototype.notify.call(this, notification);
     },
 
-    createIcon: function(size) {
-        let textureCache = St.TextureCache.get_default();
+    _setApp: function() {
+        this.app = Shell.WindowTracker.get_default().get_app_from_pid(this._pid);
+        if (!this.app)
+            return;
 
-        if (this._icon) {
-            if (this._icon.substr(0, 7) == 'file://')
-                return textureCache.load_uri_async(this._icon, size, size);
-            else if (this._icon[0] == '/') {
-                let uri = GLib.filename_to_uri(this._icon, null);
-                return textureCache.load_uri_async(uri, size, size);
-            } else
-                return textureCache.load_icon_name(this._icon, size);
-        } else if (this._iconData) {
-            let [width, height, rowStride, hasAlpha,
-                 bitsPerSample, nChannels, data] = this._iconData;
-            return textureCache.load_from_raw(data, data.length, hasAlpha,
-                                              width, height, rowStride, size);
-        } else {
-            let stockIcon;
-            switch (this._urgency) {
-                case Urgency.LOW:
-                case Urgency.NORMAL:
-                    stockIcon = 'gtk-dialog-info';
-                    break;
-                case Urgency.CRITICAL:
-                    stockIcon = 'gtk-dialog-error';
-                    break;
-            }
-            return textureCache.load_icon_name(stockIcon, size);
+        // Only override the icon if we were previously using
+        // notification-based icons (ie, not a trayicon)
+        if (this.useNotificationIcon) {
+            this.useNotificationIcon = false;
+            this._setSummaryIcon(this.app.create_icon_texture (this.ICON_SIZE));
         }
     },
 
-    clicked: function() {
-        this.openApp();
-        MessageTray.Source.prototype.clicked.call(this);
+    setTrayIcon: function(icon) {
+        this._setSummaryIcon(icon);
+        this.useNotificationIcon = false;
+        this._isTrayIcon = true;
     },
 
-    setApp: function(app) {
-        this.app = app;
-        if (this._openAppRequested)
-            this.openApp();
+    _notificationClicked: function(notification) {
+        notification.destroy();
+        this.openApp();
+        this.activated();
+    },
+
+    activated: function() {
+        if (!this._isTrayIcon)
+            this.destroy();
     },
 
     openApp: function() {
-        if (this.app == null) {
-            this._openAppRequested = true;
+        if (this.app == null)
             return;
-        }
+
         let windows = this.app.get_windows();
         if (windows.length > 0) {
             let mostRecentWindow = windows[0];
             Main.activateWindow(mostRecentWindow);
         }
-        this._openAppRequested = false;
     }
 };
