@@ -1,19 +1,15 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
-const Gio = imports.gi.Gio;
-const Gtk = imports.gi.Gtk;
 const Mainloop = imports.mainloop;
-const Shell = imports.gi.Shell;
 const Signals = imports.signals;
 const Lang = imports.lang;
 const St = imports.gi.St;
 const Gettext = imports.gettext.domain('gnome-shell');
 const _ = Gettext.gettext;
 
-const AppDisplay = imports.ui.appDisplay;
-const DocDisplay = imports.ui.docDisplay;
 const GenericDisplay = imports.ui.genericDisplay;
+const Lightbox = imports.ui.lightbox;
 const Main = imports.ui.main;
 const Panel = imports.ui.panel;
 const Dash = imports.ui.dash;
@@ -23,6 +19,9 @@ const WorkspacesView = imports.ui.workspacesView;
 // Time for initial animation going into Overview mode
 const ANIMATION_TIME = 0.25;
 
+// Time for pane menus to fade in/out
+const PANE_FADE_TIME = 0.1;
+
 // We divide the screen into a grid of rows and columns, which we use
 // to help us position the Overview components, such as the side panel
 // that lists applications and documents, the workspaces display, and 
@@ -174,13 +173,21 @@ function Overview() {
 
 Overview.prototype = {
     _init : function() {
-        this._group = new St.BoxLayout({ style_class: 'overview' });
+        this._group = new St.Group({ style_class: 'overview' });
         this._group._delegate = this;
+        this._group.connect('destroy', Lang.bind(this,
+            function() {
+                if (this._lightbox) {
+                    this._lightbox.destroy();
+                    this._lightbox = null;
+                }
+            }));
 
         this.infoBar = new InfoBar();
         this._group.add_actor(this.infoBar.actor);
 
         this._workspacesManager = null;
+        this._lightbox = null;
 
         this.visible = false;
         this.animationInProgress = false;
@@ -268,6 +275,7 @@ Overview.prototype = {
         this._recalculateGridSizes();
 
         this._group.set_position(primary.x, primary.y);
+        this._group.set_size(primary.width, primary.height);
 
         let contentY = Panel.PANEL_HEIGHT;
         let contentHeight = primary.height - contentY;
@@ -361,7 +369,11 @@ Overview.prototype = {
                         this._activeDisplayPane.close();
                     return true;
                 }));
-                this.workspaces.actor.opacity = 64;
+                if (!this._lightbox)
+                    this._lightbox = new Lightbox.Lightbox(this._group,
+                                                           { fadeTime: PANE_FADE_TIME });
+                this._lightbox.show();
+                this._lightbox.highlight(this._paneContainer);
             } else if (pane == this._activeDisplayPane) {
                 this._activeDisplayPane = null;
                 if (backgroundEventId != null) {
@@ -370,28 +382,25 @@ Overview.prototype = {
                 }
                 this._transparentBackground.lower_bottom();
                 this._paneContainer.hide();
-                this.workspaces.actor.opacity = 255;
+                this._lightbox.hide();
             }
         }));
     },
 
-    //// Draggable target interface ////
+    //// Public methods ////
 
-    // Closes any active panes if a GenericDisplayItem is being
-    // dragged over the Overview, i.e. as soon as it starts being dragged.
-    // This allows the user to place the item on any workspace.
-    handleDragOver : function(source, actor, x, y, time) {
-        if (source instanceof GenericDisplay.GenericDisplayItem
-            || source instanceof AppDisplay.AppIcon) {
+    beginItemDrag: function(source) {
+        // Close any active panes if @source is a GenericDisplayItem.
+        // This allows the user to place the item on any workspace.
+        if (source instanceof GenericDisplay.GenericDisplayItem)
             if (this._activeDisplayPane != null)
                 this._activeDisplayPane.close();
-            return true;
-        }
-
-        return false;
+        this.emit('item-drag-begin');
     },
 
-    //// Public methods ////
+    endItemDrag: function(source) {
+        this.emit('item-drag-end');
+    },
 
     // Returns the scale the Overview has when we just start zooming out
     // to overview mode. That is, when just the active workspace is showing.
