@@ -25,8 +25,7 @@
 
 #include "config.h"
 
-#define MUTTER_BUILDING_PLUGIN 1
-#include <mutter-plugin.h>
+#include <meta-plugin.h>
 
 #include <glib/gi18n-lib.h>
 
@@ -38,52 +37,59 @@
 #include <gjs/gjs.h>
 #include <girepository.h>
 #include <gmodule.h>
+#ifdef HAVE_MALLINFO
 #include <malloc.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 
+#include <GL/glx.h>
+#include <GL/glxext.h>
+
 #include "display.h"
 
 #include "shell-global-private.h"
 #include "shell-perf-log.h"
-#include "shell-wm.h"
+#include "shell-wm-private.h"
 #include "st.h"
 
 static void gnome_shell_plugin_dispose     (GObject *object);
 static void gnome_shell_plugin_finalize    (GObject *object);
 
-static void     gnome_shell_plugin_start            (MutterPlugin         *plugin);
-static void     gnome_shell_plugin_minimize         (MutterPlugin         *plugin,
-                                                     MutterWindow         *actor);
-static void     gnome_shell_plugin_maximize         (MutterPlugin         *plugin,
-                                                     MutterWindow         *actor,
-                                                     gint                  x,
-                                                     gint                  y,
-                                                     gint                  width,
-                                                     gint                  height);
-static void     gnome_shell_plugin_unmaximize       (MutterPlugin         *plugin,
-                                                     MutterWindow         *actor,
-                                                     gint                  x,
-                                                     gint                  y,
-                                                     gint                  width,
-                                                     gint                  height);
-static void     gnome_shell_plugin_map              (MutterPlugin         *plugin,
-                                                     MutterWindow         *actor);
-static void     gnome_shell_plugin_destroy          (MutterPlugin         *plugin,
-                                                     MutterWindow         *actor);
+static void gnome_shell_plugin_start            (MetaPlugin          *plugin);
+static void gnome_shell_plugin_minimize         (MetaPlugin          *plugin,
+                                                 MetaWindowActor     *actor);
+static void gnome_shell_plugin_maximize         (MetaPlugin          *plugin,
+                                                 MetaWindowActor     *actor,
+                                                 gint                 x,
+                                                 gint                 y,
+                                                 gint                 width,
+                                                 gint                 height);
+static void gnome_shell_plugin_unmaximize       (MetaPlugin          *plugin,
+                                                 MetaWindowActor     *actor,
+                                                 gint                 x,
+                                                 gint                 y,
+                                                 gint                 width,
+                                                 gint                 height);
+static void gnome_shell_plugin_map              (MetaPlugin          *plugin,
+                                                 MetaWindowActor     *actor);
+static void gnome_shell_plugin_destroy          (MetaPlugin          *plugin,
+                                                 MetaWindowActor     *actor);
 
-static void     gnome_shell_plugin_switch_workspace (MutterPlugin         *plugin,
-                                                     const GList         **actors,
-                                                     gint                  from,
-                                                     gint                  to,
-                                                     MetaMotionDirection   direction);
-static void     gnome_shell_plugin_kill_effect      (MutterPlugin         *plugin,
-                                                     MutterWindow         *actor,
-                                                     gulong                events);
+static void gnome_shell_plugin_switch_workspace (MetaPlugin          *plugin,
+                                                 gint                 from,
+                                                 gint                 to,
+                                                 MetaMotionDirection  direction);
+
+static void gnome_shell_plugin_kill_window_effects   (MetaPlugin      *plugin,
+                                                      MetaWindowActor *actor);
+static void gnome_shell_plugin_kill_switch_workspace (MetaPlugin      *plugin);
+
+
+static gboolean              gnome_shell_plugin_xevent_filter (MetaPlugin *plugin,
+                                                               XEvent     *event);
+static const MetaPluginInfo *gnome_shell_plugin_plugin_info   (MetaPlugin *plugin);
 
-static gboolean                gnome_shell_plugin_xevent_filter (MutterPlugin *plugin,
-                                                                 XEvent       *event);
-static const MutterPluginInfo *gnome_shell_plugin_plugin_info   (MutterPlugin *plugin);
 
 #define GNOME_TYPE_SHELL_PLUGIN            (gnome_shell_plugin_get_type ())
 #define GNOME_SHELL_PLUGIN(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOME_TYPE_SHELL_PLUGIN, GnomeShellPlugin))
@@ -97,30 +103,34 @@ typedef struct _GnomeShellPluginClass   GnomeShellPluginClass;
 
 struct _GnomeShellPlugin
 {
-  MutterPlugin parent;
+  MetaPlugin parent;
 
   GjsContext *gjs_context;
   Atom panel_action;
   Atom panel_action_run_dialog;
   Atom panel_action_main_menu;
+
+  int glx_error_base;
+  int glx_event_base;
+  guint have_swap_event : 1;
 };
 
 struct _GnomeShellPluginClass
 {
-  MutterPluginClass parent_class;
+  MetaPluginClass parent_class;
 };
 
 /*
  * Create the plugin struct; function pointers initialized in
  * g_module_check_init().
  */
-MUTTER_PLUGIN_DECLARE(GnomeShellPlugin, gnome_shell_plugin);
+META_PLUGIN_DECLARE(GnomeShellPlugin, gnome_shell_plugin);
 
 static void
 gnome_shell_plugin_class_init (GnomeShellPluginClass *klass)
 {
   GObjectClass      *gobject_class = G_OBJECT_CLASS (klass);
-  MutterPluginClass *plugin_class  = MUTTER_PLUGIN_CLASS (klass);
+  MetaPluginClass *plugin_class  = META_PLUGIN_CLASS (klass);
 
   gobject_class->dispose         = gnome_shell_plugin_dispose;
   gobject_class->finalize        = gnome_shell_plugin_finalize;
@@ -133,7 +143,9 @@ gnome_shell_plugin_class_init (GnomeShellPluginClass *klass)
   plugin_class->destroy          = gnome_shell_plugin_destroy;
 
   plugin_class->switch_workspace = gnome_shell_plugin_switch_workspace;
-  plugin_class->kill_effect      = gnome_shell_plugin_kill_effect;
+
+  plugin_class->kill_window_effects   = gnome_shell_plugin_kill_window_effects;
+  plugin_class->kill_switch_workspace = gnome_shell_plugin_kill_switch_workspace;
 
   plugin_class->xevent_filter    = gnome_shell_plugin_xevent_filter;
   plugin_class->plugin_info      = gnome_shell_plugin_plugin_info;
@@ -142,8 +154,13 @@ gnome_shell_plugin_class_init (GnomeShellPluginClass *klass)
 static void
 gnome_shell_plugin_init (GnomeShellPlugin *shell_plugin)
 {
+  g_setenv ("XDG_MENU_PREFIX", "gs-", TRUE);
+  meta_prefs_override_preference_location ("/apps/mutter/general/attach_modal_dialogs",
+                                           "/desktop/gnome/shell/windows/attach_modal_dialogs");
   meta_prefs_override_preference_location ("/apps/metacity/general/button_layout",
                                            "/desktop/gnome/shell/windows/button_layout");
+  meta_prefs_override_preference_location ("/apps/metacity/general/side_by_side_tiling",
+                                           "/desktop/gnome/shell/windows/side_by_side_tiling");
 }
 
 static void
@@ -238,6 +255,7 @@ static void
 malloc_statistics_callback (ShellPerfLog *perf_log,
                             gpointer      data)
 {
+#ifdef HAVE_MALLINFO
   struct mallinfo info = mallinfo ();
 
   shell_perf_log_update_statistic_i (perf_log,
@@ -249,6 +267,7 @@ malloc_statistics_callback (ShellPerfLog *perf_log,
   shell_perf_log_update_statistic_i (perf_log,
                                      "malloc.usedSize",
                                      info.uordblks);
+#endif
 }
 
 static void
@@ -279,17 +298,28 @@ add_statistics (GnomeShellPlugin *shell_plugin)
 }
 
 static void
-gnome_shell_plugin_start (MutterPlugin *plugin)
+gvc_muted_debug_log_handler (const char     *log_domain,
+                             GLogLevelFlags  log_level,
+                             const char     *message,
+                             gpointer        data)
+{
+  /* Intentionally empty to discard message */
+}
+
+static void
+gnome_shell_plugin_start (MetaPlugin *plugin)
 {
   GnomeShellPlugin *shell_plugin = GNOME_SHELL_PLUGIN (plugin);
   MetaScreen *screen;
   MetaDisplay *display;
+  Display *xdisplay;
   GtkSettings *settings;
   GError *error = NULL;
   int status;
   const char *shell_js;
   char **search_path;
   ShellGlobal *global;
+  const char *glx_extensions;
 
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -307,9 +337,24 @@ gnome_shell_plugin_start (MutterPlugin *plugin)
                     NULL);
   update_font_options (settings);
 
-  screen = mutter_plugin_get_screen (plugin);
+  screen = meta_plugin_get_screen (plugin);
   display = meta_screen_get_display (screen);
 
+  xdisplay = meta_display_get_xdisplay (display);
+
+  glXQueryExtension (xdisplay,
+                     &shell_plugin->glx_error_base,
+                     &shell_plugin->glx_event_base);
+
+  glx_extensions = glXQueryExtensionsString (xdisplay,
+                                             meta_screen_get_screen_number (screen));
+  shell_plugin->have_swap_event = strstr (glx_extensions, "GLX_INTEL_swap_event") != NULL;
+
+  shell_perf_log_define_event (shell_perf_log_get_default (),
+                               "glx.swapComplete",
+                               "GL buffer swap complete event received (with timestamp of completion)",
+                               "x");
+
   g_irepository_prepend_search_path (GNOME_SHELL_PKGLIBDIR);
 
   shell_js = g_getenv("GNOME_SHELL_JS");
@@ -320,10 +365,14 @@ gnome_shell_plugin_start (MutterPlugin *plugin)
   shell_plugin->gjs_context = gjs_context_new_with_search_path(search_path);
   g_strfreev(search_path);
 
+  /* Disable the gnome-volume-control debug */
+  g_log_set_handler ("Gvc", G_LOG_LEVEL_DEBUG,
+                     gvc_muted_debug_log_handler, NULL);
+
   /* Initialize the global object here. */
   global = shell_global_get ();
 
-  _shell_global_set_plugin (global, MUTTER_PLUGIN(shell_plugin));
+  _shell_global_set_plugin (global, META_PLUGIN(shell_plugin));
   _shell_global_set_gjs_context (global, shell_plugin->gjs_context);
 
   add_statistics (shell_plugin);
@@ -379,8 +428,8 @@ get_shell_wm (void)
 }
 
 static void
-gnome_shell_plugin_minimize (MutterPlugin         *plugin,
-			     MutterWindow         *actor)
+gnome_shell_plugin_minimize (MetaPlugin         *plugin,
+			     MetaWindowActor    *actor)
 {
   _shell_wm_minimize (get_shell_wm (),
                       actor);
@@ -388,76 +437,112 @@ gnome_shell_plugin_minimize (MutterPlugin         *plugin,
 }
 
 static void
-gnome_shell_plugin_maximize (MutterPlugin         *plugin,
-                             MutterWindow         *actor,
-                             gint                  x,
-                             gint                  y,
-                             gint                  width,
-                             gint                  height)
+gnome_shell_plugin_maximize (MetaPlugin         *plugin,
+                             MetaWindowActor    *actor,
+                             gint                x,
+                             gint                y,
+                             gint                width,
+                             gint                height)
 {
   _shell_wm_maximize (get_shell_wm (),
                       actor, x, y, width, height);
 }
 
 static void
-gnome_shell_plugin_unmaximize (MutterPlugin         *plugin,
-                               MutterWindow         *actor,
-                               gint                  x,
-                               gint                  y,
-                               gint                  width,
-                               gint                  height)
+gnome_shell_plugin_unmaximize (MetaPlugin         *plugin,
+                               MetaWindowActor    *actor,
+                               gint                x,
+                               gint                y,
+                               gint                width,
+                               gint                height)
 {
   _shell_wm_unmaximize (get_shell_wm (),
                         actor, x, y, width, height);
 }
 
 static void
-gnome_shell_plugin_map (MutterPlugin         *plugin,
-                        MutterWindow         *actor)
+gnome_shell_plugin_map (MetaPlugin         *plugin,
+                        MetaWindowActor    *actor)
 {
   _shell_wm_map (get_shell_wm (),
                  actor);
 }
 
 static void
-gnome_shell_plugin_destroy (MutterPlugin         *plugin,
-                            MutterWindow         *actor)
+gnome_shell_plugin_destroy (MetaPlugin         *plugin,
+                            MetaWindowActor    *actor)
 {
   _shell_wm_destroy (get_shell_wm (),
                      actor);
 }
 
 static void
-gnome_shell_plugin_switch_workspace (MutterPlugin         *plugin,
-                                     const GList         **actors,
-                                     gint                  from,
-                                     gint                  to,
-                                     MetaMotionDirection   direction)
+gnome_shell_plugin_switch_workspace (MetaPlugin         *plugin,
+                                     gint                from,
+                                     gint                to,
+                                     MetaMotionDirection direction)
 {
-  _shell_wm_switch_workspace (get_shell_wm(),
-                              actors, from, to, direction);
+  _shell_wm_switch_workspace (get_shell_wm(), from, to, direction);
 }
 
 static void
-gnome_shell_plugin_kill_effect (MutterPlugin         *plugin,
-                                MutterWindow         *actor,
-                                gulong                events)
+gnome_shell_plugin_kill_window_effects (MetaPlugin         *plugin,
+                                        MetaWindowActor    *actor)
 {
-  _shell_wm_kill_effect (get_shell_wm(),
-                         actor, events);
+  _shell_wm_kill_window_effects (get_shell_wm(), actor);
+}
+
+static void
+gnome_shell_plugin_kill_switch_workspace (MetaPlugin         *plugin)
+{
+  _shell_wm_kill_switch_workspace (get_shell_wm());
 }
 
 static gboolean
-gnome_shell_plugin_xevent_filter (MutterPlugin *plugin,
-                                  XEvent       *xev)
+gnome_shell_plugin_xevent_filter (MetaPlugin *plugin,
+                                  XEvent     *xev)
 {
+#ifdef GLX_INTEL_swap_event
+  GnomeShellPlugin *shell_plugin = GNOME_SHELL_PLUGIN (plugin);
+
+  if (shell_plugin->have_swap_event &&
+      xev->type == (shell_plugin->glx_event_base + GLX_BufferSwapComplete))
+    {
+      GLXBufferSwapComplete *swap_complete_event;
+      swap_complete_event = (GLXBufferSwapComplete *)xev;
+
+      /* Buggy early versions of the INTEL_swap_event implementation in Mesa
+       * can send this with a ust of 0. Simplify life for consumers
+       * by ignoring such events */
+      if (swap_complete_event->ust != 0)
+        shell_perf_log_event_x (shell_perf_log_get_default (),
+                                "glx.swapComplete",
+                                swap_complete_event->ust);
+    }
+#endif
+
+  /* When the pointer leaves the stage to enter a child of the stage
+   * (like a notification icon), we don't want to produce Clutter leave
+   * events. But Clutter treats all leave events identically, so we
+   * need hide the detail = NotifyInferior events from it.
+   *
+   * Since Clutter doesn't see any event at all, this does mean that
+   * it won't produce an enter event on a Clutter actor that surrounds
+   * the child (unless it gets a MotionNotify before the Enter event).
+   * Other weirdness is likely also possible.
+   */
+  if ((xev->xany.type == EnterNotify || xev->xany.type == LeaveNotify)
+      && xev->xcrossing.detail == NotifyInferior
+      && xev->xcrossing.window == clutter_x11_get_stage_window (CLUTTER_STAGE (clutter_stage_get_default ())))
+    return TRUE;
+
   return clutter_x11_handle_event (xev) != CLUTTER_X11_FILTER_CONTINUE;
 }
 
 static const
-MutterPluginInfo *gnome_shell_plugin_plugin_info (MutterPlugin *plugin)
+MetaPluginInfo *gnome_shell_plugin_plugin_info (MetaPlugin *plugin)
 {
-  static const MutterPluginInfo info = {
+  static const MetaPluginInfo info = {
     .name = "GNOME Shell",
     .version = "0.1",
     .author = "Various",
