@@ -3,7 +3,9 @@
 #include "config.h"
 
 #include "shell-global-private.h"
+#include "shell-enum-types.h"
 #include "shell-perf-log.h"
+#include "shell-window-tracker.h"
 #include "shell-wm.h"
 
 #include "display.h"
@@ -21,10 +23,12 @@
 #include <gio/gio.h>
 #include <math.h>
 #include <X11/extensions/Xfixes.h>
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <canberra.h>
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
+#include "shell-jsapi-compat-private.h"
 
 #define SHELL_DBUS_SERVICE "org.gnome.Shell"
 #define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
@@ -48,7 +52,7 @@ struct _ShellGlobal {
   XserverRegion input_region;
 
   GjsContext *js_context;
-  MutterPlugin *plugin;
+  MetaPlugin *plugin;
   ShellWM *wm;
   GSettings *settings;
   const char *datadir;
@@ -57,12 +61,16 @@ struct _ShellGlobal {
 
   /* Displays the root window; see shell_global_create_root_pixmap_actor() */
   ClutterActor *root_pixmap;
+  GdkWindow    *stage_window;
 
   gint last_change_screen_width, last_change_screen_height;
 
   guint work_count;
   GSList *leisure_closures;
   guint leisure_function_id;
+
+  /* For sound notifications */
+  ca_context *sound_context;
 };
 
 enum {
@@ -74,6 +82,7 @@ enum {
   PROP_SCREEN_WIDTH,
   PROP_SCREEN_HEIGHT,
   PROP_STAGE,
+  PROP_STAGE_INPUT_MODE,
   PROP_WINDOW_GROUP,
   PROP_WINDOW_MANAGER,
   PROP_SETTINGS,
@@ -90,8 +99,14 @@ shell_global_set_property(GObject         *object,
                           const GValue    *value,
                           GParamSpec      *pspec)
 {
+  ShellGlobal *global = SHELL_GLOBAL (object);
+
   switch (prop_id)
     {
+    case PROP_STAGE_INPUT_MODE:
+      shell_global_set_stage_input_mode (global, g_value_get_enum (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -109,7 +124,7 @@ shell_global_get_property(GObject         *object,
   switch (prop_id)
     {
     case PROP_OVERLAY_GROUP:
-      g_value_set_object (value, mutter_plugin_get_overlay_group (global->plugin));
+      g_value_set_object (value, meta_plugin_get_overlay_group (global->plugin));
       break;
     case PROP_SCREEN:
       g_value_set_object (value, shell_global_get_screen (global));
@@ -121,7 +136,7 @@ shell_global_get_property(GObject         *object,
       {
         int width, height;
 
-        mutter_plugin_query_screen_size (global->plugin, &width, &height);
+        meta_plugin_query_screen_size (global->plugin, &width, &height);
         g_value_set_int (value, width);
       }
       break;
@@ -129,15 +144,18 @@ shell_global_get_property(GObject         *object,
       {
         int width, height;
 
-        mutter_plugin_query_screen_size (global->plugin, &width, &height);
+        meta_plugin_query_screen_size (global->plugin, &width, &height);
         g_value_set_int (value, height);
       }
       break;
     case PROP_STAGE:
-      g_value_set_object (value, mutter_plugin_get_stage (global->plugin));
+      g_value_set_object (value, meta_plugin_get_stage (global->plugin));
+      break;
+    case PROP_STAGE_INPUT_MODE:
+      g_value_set_enum (value, global->input_mode);
       break;
     case PROP_WINDOW_GROUP:
-      g_value_set_object (value, mutter_plugin_get_window_group (global->plugin));
+      g_value_set_object (value, meta_plugin_get_window_group (global->plugin));
       break;
     case PROP_WINDOW_MANAGER:
       g_value_set_object (value, global->wm);
@@ -194,11 +212,16 @@ shell_global_init (ShellGlobal *global)
   global->gtk_grab_active = FALSE;
 
   global->root_pixmap = NULL;
+  global->stage_window = NULL;
 
   global->input_mode = SHELL_STAGE_INPUT_MODE_NORMAL;
 
   global->last_change_screen_width = 0;
   global->last_change_screen_height = 0;
+
+  ca_context_create (&global->sound_context);
+  ca_context_change_props (global->sound_context, CA_PROP_APPLICATION_NAME, PACKAGE_NAME, CA_PROP_APPLICATION_ID, "org.gnome.Shell", NULL);
+  ca_context_open (global->sound_context);
 }
 
 static void
@@ -254,6 +277,14 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         "Stage holding the desktop scene graph",
                                                         CLUTTER_TYPE_ACTOR,
                                                         G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_STAGE_INPUT_MODE,
+                                   g_param_spec_enum ("stage-input-mode",
+                                                      "Stage input mode",
+                                                      "The stage input mode",
+                                                      SHELL_TYPE_STAGE_INPUT_MODE,
+                                                      SHELL_STAGE_INPUT_MODE_NORMAL,
+                                                      G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_WINDOW_GROUP,
                                    g_param_spec_object ("window-group",
@@ -341,6 +372,32 @@ shell_global_get (void)
   return the_object;
 }
 
+static void
+focus_window_changed (MetaDisplay *display,
+                      GParamSpec  *param,
+                      gpointer     user_data)
+{
+  ShellGlobal *global = user_data;
+
+  if (global->input_mode == SHELL_STAGE_INPUT_MODE_FOCUSED &&
+      meta_display_get_focus_window (display) != NULL)
+    shell_global_set_stage_input_mode (global, SHELL_STAGE_INPUT_MODE_NORMAL);
+}
+
+static void
+shell_global_focus_stage (ShellGlobal *global)
+{
+  Display *xdpy;
+  ClutterActor *stage;
+  Window xstage;
+
+  stage = meta_plugin_get_stage (global->plugin);
+  xstage = clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
+  xdpy = meta_plugin_get_xdisplay (global->plugin);
+  XSetInputFocus (xdpy, xstage, RevertToPointerRoot,
+                  shell_global_get_current_time (global));
+}
+
 /**
  * shell_global_set_stage_input_mode:
  * @global: the #ShellGlobal
@@ -355,6 +412,12 @@ shell_global_get (void)
  * outside that region. When it is %SHELL_STAGE_INPUT_MODE_FULLSCREEN,
  * the stage absorbs all input.
  *
+ * When the input mode is %SHELL_STAGE_INPUT_MODE_FOCUSED, the pointer
+ * is handled as with %SHELL_STAGE_INPUT_MODE_NORMAL, but additionally
+ * the stage window has the keyboard focus. If the stage loses the
+ * focus (eg, because the user clicked into a window) the input mode
+ * will revert to %SHELL_STAGE_INPUT_MODE_NORMAL.
+ *
  * Note that whenever a mutter-internal Gtk widget has a pointer grab,
  * the shell behaves as though it was in
  * %SHELL_STAGE_INPUT_MODE_NONREACTIVE, to ensure that the widget gets
@@ -367,13 +430,102 @@ shell_global_set_stage_input_mode (ShellGlobal         *global,
   g_return_if_fail (SHELL_IS_GLOBAL (global));
 
   if (mode == SHELL_STAGE_INPUT_MODE_NONREACTIVE || global->gtk_grab_active)
-    mutter_plugin_set_stage_reactive (global->plugin, FALSE);
+    meta_plugin_set_stage_reactive (global->plugin, FALSE);
   else if (mode == SHELL_STAGE_INPUT_MODE_FULLSCREEN || !global->input_region)
-    mutter_plugin_set_stage_reactive (global->plugin, TRUE);
+    meta_plugin_set_stage_reactive (global->plugin, TRUE);
   else
-    mutter_plugin_set_stage_input_region (global->plugin, global->input_region);
+    meta_plugin_set_stage_input_region (global->plugin, global->input_region);
 
-  global->input_mode = mode;
+  if (mode == SHELL_STAGE_INPUT_MODE_FOCUSED)
+    shell_global_focus_stage (global);
+
+  if (mode != global->input_mode)
+    {
+      global->input_mode = mode;
+      g_object_notify (G_OBJECT (global), "stage-input-mode");
+    }
+}
+
+/**
+ * shell_global_set_cursor:
+ * @global: A #ShellGlobal
+ * @type: the type of the cursor
+ *
+ * Set the cursor on the stage window.
+ */
+void
+shell_global_set_cursor (ShellGlobal *global,
+                         ShellCursor type)
+{
+  const char *name;
+  GdkCursor *cursor;
+
+  switch (type)
+    {
+    case SHELL_CURSOR_DND_IN_DRAG:
+      name = "dnd-none";
+      break;
+    case SHELL_CURSOR_DND_MOVE:
+      name = "dnd-move";
+      break;
+    case SHELL_CURSOR_DND_COPY:
+      name = "dnd-copy";
+      break;
+    case SHELL_CURSOR_DND_UNSUPPORTED_TARGET:
+      name = "dnd-none";
+      break;
+    default:
+      g_return_if_reached ();
+    }
+
+  cursor = gdk_cursor_new_from_name (gdk_display_get_default (), name);
+  if (!cursor)
+    {
+      GdkCursorType cursor_type;
+      switch (type)
+        {
+        case SHELL_CURSOR_DND_IN_DRAG:
+          cursor_type = GDK_FLEUR;
+          break;
+        case SHELL_CURSOR_DND_MOVE:
+          cursor_type = GDK_TARGET;
+          break;
+        case SHELL_CURSOR_DND_COPY:
+          cursor_type = GDK_PLUS;
+          break;
+        case SHELL_CURSOR_DND_UNSUPPORTED_TARGET:
+          cursor_type = GDK_X_CURSOR;
+          break;
+        default:
+          g_return_if_reached ();
+        }
+      cursor = gdk_cursor_new (cursor_type);
+    }
+  if (!global->stage_window)
+    {
+      ClutterStage *stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
+
+      global->stage_window = gdk_window_foreign_new (clutter_x11_get_stage_window (stage));
+    }
+
+  gdk_window_set_cursor (global->stage_window, cursor);
+
+  gdk_cursor_unref (cursor);
+}
+
+/**
+ * shell_global_unset_cursor:
+ * @global: A #ShellGlobal
+ *
+ * Unset the cursor on the stage window.
+ */
+void
+shell_global_unset_cursor (ShellGlobal  *global)
+{
+  if (!global->stage_window) /* cursor has never been set */
+    return;
+
+  gdk_window_set_cursor (global->stage_window, NULL);
 }
 
 /**
@@ -390,7 +542,7 @@ void
 shell_global_set_stage_input_region (ShellGlobal *global,
                                      GSList      *rectangles)
 {
-  MetaScreen *screen = mutter_plugin_get_screen (global->plugin);
+  MetaScreen *screen = meta_plugin_get_screen (global->plugin);
   MetaDisplay *display = meta_screen_get_display (screen);
   Display *xdpy = meta_display_get_xdisplay (display);
   MetaRectangle *rect;
@@ -431,7 +583,7 @@ shell_global_set_stage_input_region (ShellGlobal *global,
 MetaScreen *
 shell_global_get_screen (ShellGlobal  *global)
 {
-  return mutter_plugin_get_screen (global->plugin);
+  return meta_plugin_get_screen (global->plugin);
 }
 
 /**
@@ -448,18 +600,18 @@ shell_global_get_gdk_screen (ShellGlobal *global)
 }
 
 /**
- * shell_global_get_windows:
+ * shell_global_get_window_actors:
  *
- * Gets the list of MutterWindows for the plugin's screen
+ * Gets the list of #MetaWindowActor for the plugin's screen
  *
- * Return value: (element-type MutterWindow) (transfer none): the list of windows
+ * Return value: (element-type Meta.WindowActor) (transfer none): the list of windows
  */
 GList *
-shell_global_get_windows (ShellGlobal *global)
+shell_global_get_window_actors (ShellGlobal *global)
 {
   g_return_val_if_fail (SHELL_IS_GLOBAL (global), NULL);
 
-  return mutter_plugin_get_windows (global->plugin);
+  return meta_plugin_get_window_actors (global->plugin);
 }
 
 static gboolean
@@ -468,7 +620,7 @@ update_screen_size (gpointer data)
   int width, height;
   ShellGlobal *global = SHELL_GLOBAL (data);
 
-  mutter_plugin_query_screen_size (global->plugin, &width, &height);
+  meta_plugin_query_screen_size (global->plugin, &width, &height);
 
   if (global->last_change_screen_width == width && global->last_change_screen_height == height)
     return FALSE;
@@ -531,10 +683,12 @@ global_stage_after_paint (ClutterStage *stage,
 }
 
 void
-_shell_global_set_plugin (ShellGlobal  *global,
-                          MutterPlugin *plugin)
+_shell_global_set_plugin (ShellGlobal *global,
+                          MetaPlugin  *plugin)
 {
   ClutterActor *stage;
+  MetaScreen *screen;
+  MetaDisplay *display;
 
   g_return_if_fail (SHELL_IS_GLOBAL (global));
   g_return_if_fail (global->plugin == NULL);
@@ -542,7 +696,7 @@ _shell_global_set_plugin (ShellGlobal  *global,
   global->plugin = plugin;
   global->wm = shell_wm_new (plugin);
 
-  stage = mutter_plugin_get_stage (plugin);
+  stage = meta_plugin_get_stage (plugin);
 
   g_signal_connect (stage, "notify::width",
                     G_CALLBACK (global_stage_notify_width), global);
@@ -563,6 +717,11 @@ _shell_global_set_plugin (ShellGlobal  *global,
                                "clutter.stagePaintDone",
                                "End of stage page repaint",
                                "");
+
+  screen = meta_plugin_get_screen (global->plugin);
+  display = meta_screen_get_display (screen);
+  g_signal_connect (display, "notify::focus-window",
+                    G_CALLBACK (focus_window_changed), global);
 }
 
 void
@@ -592,10 +751,10 @@ gboolean
 shell_global_begin_modal (ShellGlobal *global,
                           guint32      timestamp)
 {
-  ClutterStage *stage = CLUTTER_STAGE (mutter_plugin_get_stage (global->plugin));
+  ClutterStage *stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
   Window stagewin = clutter_x11_get_stage_window (stage);
 
-  return mutter_plugin_begin_modal (global->plugin, stagewin, None, 0, timestamp);
+  return meta_plugin_begin_modal (global->plugin, stagewin, None, 0, timestamp);
 }
 
 /**
@@ -608,7 +767,7 @@ void
 shell_global_end_modal (ShellGlobal *global,
                         guint32      timestamp)
 {
-  mutter_plugin_end_modal (global->plugin, timestamp);
+  meta_plugin_end_modal (global->plugin, timestamp);
 }
 
 /**
@@ -629,7 +788,7 @@ shell_global_end_modal (ShellGlobal *global,
 gboolean
 shell_global_display_is_grabbed (ShellGlobal *global)
 {
-  MetaScreen *screen = mutter_plugin_get_screen (global->plugin);
+  MetaScreen *screen = meta_plugin_get_screen (global->plugin);
   MetaDisplay *display = meta_screen_get_display (screen);
 
   return meta_display_get_grab_op (display) != META_GRAB_OP_NONE;
@@ -660,6 +819,8 @@ shell_global_add_extension_importer (ShellGlobal *global,
   JSContext *context = gjs_context_get_native_context (global->js_context);
   char *search_path[2] = { 0, 0 };
 
+  JS_BeginRequest (context);
+
   // This is a bit of a hack; ideally we'd be able to pass our target
   // object directly into this function, but introspection doesn't
   // support that at the moment.  Instead evaluate a string to get it.
@@ -679,18 +840,22 @@ shell_global_add_extension_importer (ShellGlobal *global,
                   G_IO_ERROR_FAILED,
                   "%s", message ? message : "(unknown)");
       g_free(message);
-      return FALSE;
+      goto out_error;
     }
 
   if (!JSVAL_IS_OBJECT (target_object))
     {
       g_error ("shell_global_add_extension_importer: invalid target object");
-      return FALSE;
+      goto out_error;
     }
 
   search_path[0] = (char*)directory;
   importer = gjs_define_importer (context, JSVAL_TO_OBJECT (target_object), target_property, (const char **)search_path, FALSE);
+  JS_EndRequest (context);
   return TRUE;
+ out_error:
+  JS_EndRequest (context);
+  return FALSE;
 }
 
 /* Code to close all file descriptors before we exec; copied from gspawn.c in GLib.
@@ -1097,7 +1262,7 @@ shell_global_create_root_pixmap_actor (ShellGlobal *global)
       /* Initialize to the stage color, since that's what will be seen
        * in the main view if there's no actual background window.
        */
-      stage = mutter_plugin_get_stage (global->plugin);
+      stage = meta_plugin_get_stage (global->plugin);
       clutter_stage_get_color (CLUTTER_STAGE (stage), &stage_color);
       clutter_texture_set_from_rgb_data (CLUTTER_TEXTURE (global->root_pixmap),
                                          /* ClutterColor has the same layout
@@ -1141,7 +1306,7 @@ shell_global_create_root_pixmap_actor (ShellGlobal *global)
  *
  * Gets a list of the bounding boxes of the active screen's monitors.
  *
- * Return value: (transfer full) (element-type GdkRectangle): a list
+ * Return value: (transfer full) (element-type Meta.Rectangle): a list
  * of monitor bounding boxes.
  */
 GSList *
@@ -1152,17 +1317,11 @@ shell_global_get_monitors (ShellGlobal *global)
   MetaRectangle rect;
   int i;
 
-  g_assert (sizeof (MetaRectangle) == sizeof (GdkRectangle) &&
-            G_STRUCT_OFFSET (MetaRectangle, x) == G_STRUCT_OFFSET (GdkRectangle, x) &&
-            G_STRUCT_OFFSET (MetaRectangle, y) == G_STRUCT_OFFSET (GdkRectangle, y) &&
-            G_STRUCT_OFFSET (MetaRectangle, width) == G_STRUCT_OFFSET (GdkRectangle, width) &&
-            G_STRUCT_OFFSET (MetaRectangle, height) == G_STRUCT_OFFSET (GdkRectangle, height));
-
   for (i = meta_screen_get_n_monitors (screen) - 1; i >= 0; i--)
     {
       meta_screen_get_monitor_geometry (screen, i, &rect);
       monitors = g_slist_prepend (monitors,
-                                  g_boxed_copy (GDK_TYPE_RECTANGLE, &rect));
+                                  meta_rectangle_copy (&rect));
     }
   return monitors;
 }
@@ -1176,11 +1335,12 @@ shell_global_get_monitors (ShellGlobal *global)
  *
  * Return value: the bounding box of the primary monitor
  */
-GdkRectangle *
+MetaRectangle *
 shell_global_get_primary_monitor (ShellGlobal  *global)
 {
   GdkScreen *screen = shell_global_get_gdk_screen (global);
-  GdkRectangle rect;
+  GdkRectangle gdk_rect;
+  MetaRectangle rect;
   gint primary = 0;
 
   /* gdk_screen_get_primary_monitor is only present in gtk-2.20+
@@ -1211,9 +1371,13 @@ shell_global_get_primary_monitor (ShellGlobal  *global)
   primary = gdk_screen_get_primary_monitor (screen);
 #endif
 
-  gdk_screen_get_monitor_geometry (screen, primary, &rect);
+  gdk_screen_get_monitor_geometry (screen, primary, &gdk_rect);
+  rect.x = gdk_rect.x;
+  rect.y = gdk_rect.y;
+  rect.width = gdk_rect.width;
+  rect.height = gdk_rect.height;
 
-  return g_boxed_copy (GDK_TYPE_RECTANGLE, &rect);
+  return meta_rectangle_copy (&rect);
 }
 
 /**
@@ -1225,7 +1389,7 @@ shell_global_get_primary_monitor (ShellGlobal  *global)
  *
  * Return value: the bounding box of the focus monitor
  */
-GdkRectangle *
+MetaRectangle *
 shell_global_get_focus_monitor (ShellGlobal  *global)
 {
   MetaScreen *screen = shell_global_get_screen (global);
@@ -1247,12 +1411,12 @@ shell_global_get_focus_monitor (ShellGlobal  *global)
           if (rect.x <= wrect.x && rect.y <= wrect.y &&
               rect.x + rect.width > wrect.x &&
               rect.y + rect.height > wrect.y)
-            return g_boxed_copy (GDK_TYPE_RECTANGLE, &rect);
+            return meta_rectangle_copy (&rect);
         }
     }
 
   meta_screen_get_monitor_geometry (screen, 0, &rect);
-  return g_boxed_copy (GDK_TYPE_RECTANGLE, &rect);
+  return meta_rectangle_copy (&rect);
 }
 
 /**
@@ -1304,7 +1468,7 @@ shell_get_event_state (ClutterEvent *event)
  *
  * Get the global GSettings instance.
  *
- * Return value: (transfer none) The GSettings object (transfer none).
+ * Return value: (transfer none): The GSettings object
  */
 GSettings *
 shell_global_get_settings (ShellGlobal *global)
@@ -1402,13 +1566,13 @@ shell_global_get_current_time (ShellGlobal *global)
 }
 
 /**
- * shell_global_get_app_launch_context:
+ * shell_global_create_app_launch_context:
  * @global: A #ShellGlobal
  *
  * Create a #GAppLaunchContext set up with the correct timestamp, and
  * targeted to activate on the current workspace.
  *
- * Return value: A new #GAppLaunchContext
+ * Return value: (transfer full): A new #GAppLaunchContext
  */
 GAppLaunchContext *
 shell_global_create_app_launch_context (ShellGlobal *global)
@@ -1455,7 +1619,8 @@ shell_global_set_property_mutable (ShellGlobal *global,
   jsuint attrs;
   JSBool found;
 
-  JS_AddRoot (context, &val);
+  JS_BeginRequest (context);
+  JS_AddValueRoot (context, &val);
 
   parts = g_strsplit (object, ".", -1);
   obj = JS_GetGlobalObject (context);
@@ -1464,30 +1629,31 @@ shell_global_set_property_mutable (ShellGlobal *global,
       if (!JS_GetProperty (context, obj, parts[i], &val))
         {
           g_strfreev (parts);
-          JS_RemoveRoot (context, &val);
-          gjs_log_exception (context, NULL);
-          return FALSE;
+          goto out_error;
         }
       obj = JSVAL_TO_OBJECT (val);
     }
   g_strfreev (parts);
 
   if (!JS_GetPropertyAttributes (context, obj, property, &attrs, &found) || !found)
-    {
-      JS_RemoveRoot (context, &val);
-      gjs_log_exception (context, NULL);
-      return FALSE;
-    }
+    goto out_error;
 
   if (mutable)
     attrs &= ~(JSPROP_PERMANENT | JSPROP_READONLY);
   else
     attrs |= (JSPROP_PERMANENT | JSPROP_READONLY);
 
-  JS_SetPropertyAttributes (context, obj, property, attrs, &found);
+  if (!JS_SetPropertyAttributes (context, obj, property, attrs, &found))
+    goto out_error;
 
-  JS_RemoveRoot (context, &val);
-  return !gjs_log_exception (context, NULL);
+  JS_RemoveValueRoot (context, &val);
+  JS_EndRequest (context);
+  return TRUE;
+ out_error:
+  gjs_log_exception (context, NULL);
+  JS_RemoveValueRoot (context, &val);
+  JS_EndRequest (context);
+  return FALSE;
 }
 
 typedef struct
@@ -1630,3 +1796,18 @@ shell_global_run_at_leisure (ShellGlobal         *global,
   if (global->work_count == 0)
     schedule_leisure_functions (global);
 }
+
+/**
+ * shell_global_play_theme_sound:
+ * @global: the #ShellGlobal
+ * @name: the sound name
+ *
+ * Plays a simple sound picked according to Freedesktop sound theme.
+ * Really just a workaround for libcanberra not being introspected.
+ */
+void
+shell_global_play_theme_sound (ShellGlobal *global,
+                               const char  *name)
+{
+  ca_context_play (global->sound_context, 0, CA_PROP_EVENT_ID, name, NULL);
+}
