@@ -24,6 +24,22 @@ st_table_child_options_get_type(void) {
 /* enumerations from "st/st-texture-cache.h" */
 #include "st/st-texture-cache.h"
 GType
+st_icon_type_get_type(void) {
+  static GType enum_type_id = 0;
+  if (G_UNLIKELY (!enum_type_id))
+    {
+      static const GEnumValue values[] = {
+        { ST_ICON_SYMBOLIC, "ST_ICON_SYMBOLIC", "symbolic" },
+        { ST_ICON_FULLCOLOR, "ST_ICON_FULLCOLOR", "fullcolor" },
+        { ST_ICON_APPLICATION, "ST_ICON_APPLICATION", "application" },
+        { ST_ICON_DOCUMENT, "ST_ICON_DOCUMENT", "document" },
+        { 0, NULL, NULL }
+      };
+      enum_type_id = g_enum_register_static (g_intern_static_string ("StIconType"), values);
+    }
+  return enum_type_id;
+}
+GType
 st_texture_cache_policy_get_type(void) {
   static GType enum_type_id = 0;
   if (G_UNLIKELY (!enum_type_id))
@@ -88,6 +104,22 @@ st_text_decoration_get_type(void) {
   return enum_type_id;
 }
 GType
+st_text_align_get_type(void) {
+  static GType enum_type_id = 0;
+  if (G_UNLIKELY (!enum_type_id))
+    {
+      static const GEnumValue values[] = {
+        { ST_TEXT_ALIGN_LEFT, "ST_TEXT_ALIGN_LEFT", "left" },
+        { ST_TEXT_ALIGN_CENTER, "ST_TEXT_ALIGN_CENTER", "center" },
+        { ST_TEXT_ALIGN_RIGHT, "ST_TEXT_ALIGN_RIGHT", "right" },
+        { ST_TEXT_ALIGN_JUSTIFY, "ST_TEXT_ALIGN_JUSTIFY", "justify" },
+        { 0, NULL, NULL }
+      };
+      enum_type_id = g_enum_register_static (g_intern_static_string ("StTextAlign"), values);
+    }
+  return enum_type_id;
+}
+GType
 st_gradient_type_get_type(void) {
   static GType enum_type_id = 0;
   if (G_UNLIKELY (!enum_type_id))
