@@ -1018,9 +1018,9 @@ st_box_layout_style_changed (StWidget *self)
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (self)->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (self);
   int old_spacing = priv->spacing;
-  double spacing = 0;
+  double spacing;
 
-  st_theme_node_get_length (theme_node, "spacing", FALSE, &spacing);
+  spacing = st_theme_node_get_length (theme_node, "spacing");
   priv->spacing = (int)(spacing + 0.5);
   if (priv->spacing != old_spacing)
     clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
